// This program reads and hosts a temperature sensor
// Make sure to define the appropriate access point and password
// The internal IP address will be displayed on the LCD, type this value into
// your browser appended with a :80 
#define _SUPPRESS_PLIB_WARNING
#include "System.h"	/* generic header for PIC32MX family */
#include "adc.h"
#include "CONU2.h"          /* User funct/params, such as InitApp             */
#include "CONU1.h"          /* User funct/params, such as InitApp             */
#include "lcd.h"            /* LCD library*/
#include <plib.h>           /* Include to use PIC32 peripheral libraries      */
#include <string.h>
#include <stdio.h>
#include "delay.h"
#include "espinitializer.h"

#define SSID "RedRover"
#define password " "

#define iend 25	//delay loop is iend times jend instruction cycles
#define jend 1000
#define led LATBbits.LATB5
unsigned char connection=0;             //If connected to wifi
char count[] = {'0','0','0','0', '0'};
int adcvalue =0;
float temp;
char tempvalue[5];
char s[200];
int u = 0;
unsigned char got_rx =0;		//rx char flag
//note only valid pins are 2,3,4,5,6,7,24,25,26-> PIC32MX250F128B
//note 4,5 are used for ICSP, 2 is USer LED is J3 is used
int pin =26; 

void clrstr(char *s);
void host(void);
void ReadTemp(void);
void togglePin(void);

void __ISR(_UART_2_VECTOR, ipl2) UART2BHandler(void) {
    s[u]= (char) (U2RXREG & 0x00ff);
    u++;
    got_rx=1;
    // Clear the UART RX interrupt flag bit
    IFS1bits.U2RXIF = 0;
}
			
int main ( void ) {
    PPSInput(2, U2RX, RPB8); //Assign U2RX to pin RPB11
    PPSOutput(4, RPB9, U2TX); //Assign U2TX to pin RPB10
    PPSInput(3, U1RX, RPB6); //Assign U1RX to pin RPB2
    PPSOutput(1, RPB7, U1TX); //Assign U1TX to pin RPB3
    //ANSELA = 0; //make sure analog is cleared
    //ANSELB = 0;
    
    //setup LED
    LATBbits.LATB5 =0; //Clear bit 5 to ensure light is off.
    TRISBbits.TRISB5 =0; //Set bit 5 to ouput

    INTEnableSystemMultiVectoredInt();
    // interrupt setting
    IEC1bits.U2RXIE = 1;
    IPC9bits.U2IP = 2; //set priority to 2
    IFS1bits.U2RXIF = 0;

    // Initialize and write to LCD
    const char Message1[] = "ESP8266 Temp";
    const char Message2[] = "Sensor";
   

    LCD_Initialize(); //initialize
    clear_display(); //clear the LCD display

    position_cursor(1); //position cursor to row 1
    write_string_LCD((char*) Message1); //outputs Message1 to display

    position_cursor(20); //position cursor to row 2
    write_string_LCD((char*) Message2); //outputs Message 2 to display

    initU2(); //initialize serial communication with ESP8266 radio
    initU1(); //initialize serial communication with terminal
    ESPinit(&SSID, &password, &s);
    got_rx=0;
    
        /*  endless loop*/
    while (1) {
        if (got_rx == 1) {
            got_rx = 0;
            if (strstr(s, "IPD") != NULL) {   // if the string contains the starting transmission
                delay(100); //wait for the rest of the string to arrive
                /*if (strstr(s, "pin") != NULL) {
                    togglePin();
                    clrstr(s);
                } else {*/
                    ReadTemp(); //measure the temperature
                    clrstr(s); //clear input string
                    putsU1("query received");
                    host();
                //}

            }
        }
    }
        Nop(); //do something else
    
}	


void host(void)
{
    
    char website[2000];
    clrstr(website); // create the html string
    strncpy(website,"<!DOCTYPE html><html>"\
            ,strlen("<!DOCTYPE html><html>"));
    strcat(website,"<body style=\"background-color:lightblue\">");
    strcat(website,"<h1 style=\"text-align:center\">Temperature Sensor Demo</h1>");
    strcat(website,"<p style=\"text-align:center\">The temperature is:");
    strcat(website,tempvalue); 
    strcat(website,"C</p>");
    strcat(website,"<p align=\"center\"><img src=\"http://www.cs.cornell.edu/Info/People/land/land.gif\" alt=\"Bruce Land\" width=\"197\" height=\"293\"></p>");
    strcat(website,"<p align=\"center\"><button onclick=location.reload(true)>Get Temperature</button>");
   
    /*strcat(website,"<button id=\"1\" class=\"led\">LED on</button><button id=\"2\" class=\"led\">LED off</button></p>");
    strcat(website,"<script src=\"http://ajax.googleapis.com/ajax/libs/jquery/2.1.3/jquery.min.js\"></script>");
    strcat(website,"<script type=\"text/javascript\">$(document).ready(function(){");
    strcat(website,"$(\".led\").click(function(){");
    strcat(website,"var p = $(this).attr('id');");
    strcat(website,"$.get(\"http://10.0.1.2:80/\", {pin:p});});});</script>");*/

    strcat(website,"</body></html>");
    
    
    
    // find the length of string and convert it to a char
    int len=0;
    len=strlen(website);
    char lenChar[5];
    sprintf(lenChar,"%d",len); 
    
    
    putsU1("this string will be printed:");
    putsU1(website);
    
    char sendNum[25];
    strncpy(sendNum,"AT+CIPSEND=0,",strlen("AT+CIPSEND=0,"));
    strcat(sendNum,lenChar);
    clrstr(s);
    putsU1(sendNum);
    delay(100);
    putsU2(sendNum); //send the string length
    delay(100);
    putsU1("return after CIPSEND");
    delay(100);
    putsU1(s);
    delay(100);

    //wait for the module to be ready to receive communication 
    do {
        delay(500);
        putsU1(s);
        if (strstr(s, ">") != NULL) {
            putsU1("Sending...");
            break;
        } 
        else if (strstr(s, "link is not") != NULL) {
            //initialize server connection on port 80
            putsU2("AT+CIPSERVER=1,80");
            do {
                delay(100);
                if (strstr(s, "OK") != NULL) {
                    putsU1("ESP Initialized as AP");
                    break;
                } else {
                    putsU1("problem setting ESP as AP");
                    break;
                }
            } while (1);
            clrstr(s);
        }
        else 
        {
            clrstr(s);
            /*putsU2(sendNum); 
            delay(100);*/
            break;
        }
    } while (1);
    clrstr(s);
    
 
    putsU2(website); //write the HTML to the ESP8266
    delay(200);
    putsU1("text sent");
    delay(100);
    putsU2("AT+CIPCLOSE=0");
    delay(200);
    putsU1("closed connection");
    delay(100);
    
    clrstr(s); 
    clrstr(website);
    clrstr(sendNum);
    clrstr(lenChar);
    delay(100);
}



void clrstr(char *s)
{
    while( *s)       // loop until *s == '\0' the  end of the string
        *s++='\0';	//clear character array
    u = 0;
}


//reads the temperature and updates the tempvalue variable
void ReadTemp(void){
   // Take an ADC reading on pin 

	initADC(pin); //Initialize ADC for AN0 pin 2 
    adcvalue=averagevalue(); //read and average 4 samples
    temp = (float) adcvalue * 330.0 / 1024.0;
    sprintf(tempvalue, "%.1f", temp);
    
    //write the temperature out to the LCD
    //clear_display(); //clear the LCD display
    position_cursor(32); //position cursor to row 1
    write_string_LCD((char *) "temp:     C");
    position_cursor(38);
    write_string_LCD((char*) tempvalue);
}

//can be used if the LED button code is used
/*void togglePin(void){
    char * pinlocation;
    pinlocation=strstr(s,"pin");
    //putsU1(pinlocation+4);
    if (*(pinlocation+4)=='1'){
        putsU1("led on");
        led=1;
    }
    else{
        putsU1("led off");
        led=0;
    }
    delay(100);
}*/