/*
** CONIO.C
** Basic CONSOLE I/O for Microstick II UART 2 board
** This code allows use of UART1 which puts data to the debug termial
*/
#ifdef __XC32
    #include <xc.h>          /* Defines special funciton registers, CP0 regs  */
    #include <p32xxxx.h>
#endif

#define _DISABLE_OPENADC10_CONFIGPORT_WARNING
#define _SUPPRESS_PLIB_WARNING
#include <plib.h>
#include "CONU1.h"
#define FPB 8000000
#define BAUDRATE 9600
#define BRGVAL ((FPB/BAUDRATE)/16)-1 //=51.083
#define U_ENABLE 0x8000		// enable the UART
#define U_TXRX	 0x1400		// enable transmission and reception
// init the serial port (UART1, 9600, 8, N, 1 )
void initU1( void)
{
	U1BRG = BRGVAL; // BAUD Rate Setting for 9600
	U1MODE 	= U_ENABLE;
	U1STA 	= U_TXRX; //set transmit polarity inverse and tranmit break bit
	U1MODEbits.BRGH = 0; // standard mode
} // initCon

// send a character to the UART1 serial port
int putU1( int c)
{
	while ( U1STAbits.UTXBF);   // wait while Tx buffer full
	U1TXREG = c;
	return c;
} // putU1
// wait for a new character to arrive to the UART1 serial port -don't use during interrupts


void putsU1point( char *s, int i)
{
    int count=0;
    while (count<i) {
        putU1(*s++); //put new character
        count++; // increment counter
    }
    putU1('\r');
    putU1('\n');

} // putsU1


char getU1( void)
{
  	while ( !U1STAbits.URXDA);	// wait for a new character to arrive
	return U1RXREG;		// read the character from the receive buffer
}// getU1

void putsU1( char *s)
{
	while( *s)			// loop until *s == '\0' the  end of the string
		putU1( *s++);	// send the character and point to the next one 
    putU1('\r'); // terminate with a cr / line feed
    putU1('\n');
} // putsU1

char *getsn1( char *s, int len)
{
    char *p = s;            // copy the buffer pointer
    do{
        *s = getU1();       // wait for a new character
        putU1( *s);         // echo character

        if (( *s==BACKSPACE)&&( s>p))
        {
            putU1( ' ');     // overwrite the last character
            putU1( BACKSPACE);
            len++;
            s--;            // back the pointer
            continue;
        }
        if ( *s=='\n')      // line feed, ignore it
            continue;
        if ( *s=='\r')      // end of line, end loop
            break;
        s++;                // increment buffer pointer
        len--;
    } while ( len>1 );      // until buffer full

    *s = '\0';              // null terminate the string

    return p;               // return buffer pointer
} // getsn1

