//This code reads weather data from a the internet using the ESP8266
//Make sure to define the appropriate network and password 
/******************************************************************************/
/*  Files to Include                                                          */
/******************************************************************************/
#ifdef __XC32
    #include <xc.h>          /* Defines special funciton registers, CP0 regs  */
    #include <p32xxxx.h>
#endif

#define _DISABLE_OPENADC10_CONFIGPORT_WARNING
#define _SUPPRESS_PLIB_WARNING
#include <plib.h>           /* Include to use PIC32 peripheral libraries      */
#include <stdint.h>         /* For uint32_t definition                        */
#include <stdbool.h>        /* For true/false definition                      */

#include <string.h>
#include <stdio.h>

#include "system.h"         /* System funct/params, like osc/periph config    */
#include "CONU2.h"          /* User funct/params, such as InitApp             */
#include "CONU1.h"          /* User funct/params, such as InitApp             */
#include "lcd.h"            /* LCD library*/

//***************************************************/
// Define Constants Variables
//***************************************************/
//#define SSID "Charlene's Wi-Fi Network" //enter you network
//#define password "sammyla1" //enter your password

#define SSID "RedRover"
#define password " "

#define buffer_size 128


/* i.e. uint32_t <variable_name>; */

/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/

//***************************************************

static char	rxchar='0'; 		//received character
char s[buffer_size];
char data[2000];
int u=0;

char cmd[100];


unsigned char got_rx =0;		//rx char flag
unsigned char connection=0;             //If connected to wifi

char *a;

//RX U2 interrupt

void __ISR(_UART_2_VECTOR, ipl2) UART2BHandler(void) { //if character becomes available
    s[u]= (char) (U2RXREG & 0x00ff); //read character and store it in an array
    u++;
    // Clear the UART RX interrupt flag bit
    IFS1bits.U2RXIF = 0;
}


void clrstr(char *s)
{
    while( *s)          // loop until *s == '\0' the  end of the string
        *s++='\0';	//clear character array
    u = 0;
}

void delay(int time) { // time 2000 =2 sec

    int i, j;
    for (i = 0; i < 800; i++)
        for (j = 0; j < time; j++);
}

void ESPinit(void) {
    clrscr1();
    putsU1("ready to rummble");

    putsU2("AT+RST");
    do {
        delay(2000);
        if (strstr(s, "OK") != NULL) {
            putsU1("ESP8266 Reset & Ready");
            clear_display(); //clear the LCD display
            position_cursor(0); //position cursor to row 1
            write_string_LCD((char*) "Reset & Ready"); //outputs Message 
            break;
        } else if (strstr(s, "FAIL") != NULL) {
            putsU1("Reset Failed");
            clear_display(); //clear the LCD display
            position_cursor(0); //position cursor to row 2
            write_string_LCD((char*) "Reset Failed"); //outputs Message 
            break;
        }
    } while (1);
    clrstr(s);



    putsU2("AT+CWJAP?"); // check if already connected to AP
    do {
        delay(2000);
        putsU1(s);
        if (strstr(s, SSID) != NULL) {
            putsU1("already connected to wifi ap");
            //clear_display(); //clear the LCD display
            position_cursor(16); //position cursor to row 1
            write_string_LCD((char*) "Alrdy connected to wifi ap"); //outputs Message
            connection = 1;
            break;
        } else if (strstr(s, "ERROR") != NULL) {
            putsU1("connecting to access point...");
            //clear_display(); //clear the LCD display
            position_cursor(16); //position cursor to row 1
            write_string_LCD((char*) "connecting to   access point..."); //outputs Message
            connection = 0;
            break;
        }
    } while (1);
    clrstr(s);

    







    if (connection == 0) { //if not connected

        strcat(cmd, "AT+CWJAP=\"");
        strcat(cmd, SSID);
        strcat(cmd, "\",\"");
        strcat(cmd, password);
        strcat(cmd, "\"");
        putsU2(cmd);

        do {
            delay(1000);
            putsU1(s);
            if (strstr(s, "OK") != NULL) {
                putsU1("connected to wifi ap");
                clear_display(); //clear the LCD display
                position_cursor(0); //position cursor to row 1
                write_string_LCD((char*) "WiFi Connected!"); //outputs Message
                connection = 1;
                break;
            } else if (strstr(s, "FAIL") != NULL) {
                //putsU2("AT+CWQAP"); //quite access point
                putsU1("failed to connect, let me try again");
                clear_display(); //clear the LCD display
                position_cursor(0); //position cursor to row 1
                write_string_LCD((char*) "Connect Failed  Let me try again"); //outputs Message
                putsU2("AT+CWQAP");
                delay(500);
                clrstr(s);
                //putsU2("AT+CWJAP=\"Charlene's Wi-Fi Network\",\"sammyla1\"");
                putsU2(cmd);
                //break;
            }
        } while (1);
        clrstr(s);

    }





    putsU2("AT+CWMODE=3");
    do {
        delay(100);
        if (!(strstr(s, "OK") || !(strstr(s, "no change")))) {
            //if ((strstr(s, "OK") || strstr(s, "no change")) != NULL) {
            putsU1("Mode set");
            break;
        } else {
            putsU1("problem setting mode");
            break;
        }
    } while (1);
    clrstr(s);




    putsU2("AT+CIPMUX=1");
    do {
        delay(100);
        if (strstr(s, "OK") != NULL) {
            putsU1("Mux set");
            break;
        } else {
            putsU1("problem setting mode");
            break;
        }
    } while (1);
    clrstr(s);



    do {
        clrstr(s);
        putsU2("AT+CIPSTART=4,\"TCP\",\"api.openweathermap.org\",80");
        delay(1000);
        if (strstr(s, "OK") != NULL) {
            break;
        } else {
            putsU1("problem connecting to api.openweathermap");

            //break;
        }
    } while (1);
    clrstr(s);



    do {
        clrstr(s);
        putsU2("AT+CIPSEND=4,91");
        delay(1000);
        if (strstr(s, ">") != NULL) {
            break;
        } else {
            putsU1("problem with CIPSEND");
            //break;
        }
    } while (1);
    clrstr(s);


    putsU2("GET /data/2.5/station/find?lat=42&lon=-76&cnt=30 HTTP/1.0\r\nHost: api.openweathermap.org\r\n");
    //delay(2000);
    putsU1("put get statement");
    clrstr(s);

    putsU1("waiting for data");
    clear_display(); //clear the LCD display
    position_cursor(0); //position cursor to row 1
    write_string_LCD((char*) "Waiting for Data"); //outputs Message
    delay(2000);
    putsU1(s);

    a = strstr(s, "temp") + 6;

    char temperature[6];
    strncpy(temperature, a, 5);
    putsU1(temperature);
    int tempnum;
    tempnum = atoi(temperature) - 273;

    char tmp[10];
    sprintf(tmp, "%d", tempnum);

    clear_display(); //clear the LCD display
    position_cursor(0); //position cursor to row 1
    write_string_LCD((char *) "temp:   C");
    position_cursor(6);
    write_string_LCD((char*) tmp); //outputs Message1 to display
    putsU1(tmp);
    LCDWrite(0x0C, 0); //turn cursor off

    char * hum;
    hum = strstr(s, "humidity") + 10;
    position_cursor(16);
    write_string_LCD((char*) "humidity:   %");
    position_cursor(26);
    write_array_LCD((char*) hum, 2);

    char * windspeed;
    windspeed = strstr(s, "wind") + 15;
    position_cursor(32);
    write_string_LCD((char*) "windspeed:   mps");
    position_cursor(43);
    write_array_LCD((char*) windspeed, 2);
}


/*********************************
    main entry point
 *********************************/
int main(void) { /* 	Initialize PPS */

    PPSInput(2, U2RX, RPB8); //Assign U2RX to pin RPB11
    PPSOutput(4, RPB9, U2TX); //Assign U2TX to pin RPB10
    PPSInput(3, U1RX, RPB6); //Assign U1RX to pin RPB2
    PPSOutput(1, RPB7, U1TX); //Assign U1TX to pin RPB3
    ANSELA = 0; //make sure analog is cleared
    ANSELB = 0;

    INTEnableSystemMultiVectoredInt();
    // interrupt setting
    IEC1bits.U2RXIE = 1;
    IPC9bits.U2IP = 2; //set priority to 2
    IFS1bits.U2RXIF = 0;

    // Initilize and write to LCD
    const char Message1[] = "ESP8266 Weather";
    const char Message2[] = "Station";


    LCD_Initialize(); //initialize
    clear_display(); //clear the LCD display

    position_cursor(1); //position cursor to row 1
    write_string_LCD((char*) Message1); //outputs Message1 to display

    position_cursor(20); //position cursor to row 2
    write_string_LCD((char*) Message2); //outputs Message 2 to display



    //Initalize WiFi chip

    initU2(); //initialize serial communication with ESP8266 radio
    initU1(); //initialize serial communication with terminal
    rxchar = 0;
    got_rx = 0;

    ESPinit(); //send commands to initialize ESP8266


    /*  endless loop*/
    while (1) {
        if (got_rx == 1) {
            got_rx = 0;
            putU2(rxchar);
        }
        Nop(); //do something else
    }
}
